package elements;


import java.time.LocalDate;
import java.time.ZoneId;
import java.util.*;

/**
 * Created by Guillermo Serrahima on 4/21/16.
 */
public class Chat {
    private static final String DATE_TAG = "Date=";
    private static final String WINNER_TAG = "Winner=";

    public static final String MODE_NORMAL = "mode_normal";
    public static final String MODE_RANDOM_HOUR = "mode_random_hour";
    public static final String MODE_COLORS = "mode_colors";
    public static final String MODE_FAR_WEST = "mode_far_west";

    private Map<String, Integer> users;
    private LocalDate lastPole;
    private String poleWinner;
    private String nextMode;

    public Chat() {
        users = new HashMap<String, Integer>();
        lastPole = null;
        poleWinner = null;
        nextMode = null;
    }

    public boolean exists(String user) {
        return users.containsKey(user);
    }

    public boolean isModeChosen() {
        return (nextMode != null);
    }

    public void add(String user) {
        users.put(user, 0);
    }

    public void add(String user, int p) {
        users.put(user, p);
    }

    /*
    public void winPole(String user, int date) throws PoleBlockedException {
        if(!users.containsKey(user)) add(user);
        if(!availablePole())  {
            if(poleWinner != null) throw new PoleBlockedException(poleWinner);
            else throw new PoleBlockedException();
        }
        users.replace(user, users.get(user)+1);

        Date d = new Date((new Long(date))*1000);
        lastPole = d.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();

        poleWinner = user;
    }
    */

    public void choseMode(String mode) {
        nextMode = mode;
    }

    public boolean availablePole() {
        LocalDate today = LocalDate.now();

        return (lastPole == null || today.isAfter(lastPole));
    }

    public String getRanking() {
        List<Map.Entry<String, Integer>> top;
        String ranking = "";
        Map.Entry<String, Integer>[] ts = new Map.Entry[users.size()];

        top = Arrays.asList((users.entrySet().toArray(ts)));
        top.sort(new UserComparator());

        for(int i = 0; i < top.size(); i++) {
            ranking += "[" + (i+1) + ".] " + getUsername(top.get(i)) + ": ";
            ranking += getPoles(top.get(i)) + "\n";
        }

        return ranking;
    }

    public String toTxt() {
        String s = DATE_TAG;
        if(lastPole == null) s += "null\n";
        else s += lastPole + "\n";

        s += WINNER_TAG;
        if(poleWinner == null) s += "null\n";
        else s += poleWinner + "\n";

        Iterator it = users.entrySet().iterator();

        while(it.hasNext()) {
            Map.Entry<String, Integer> entry = (Map.Entry<String, Integer>) it.next();
            s += getUsername(entry) + ":" + getPoles(entry) + "\n";
        }

        return s;
    }

    public static Chat fromTxt(List<String> lines) {
        Chat c = new Chat();
        int i = 0;
        System.out.println("Reading " + lines.get(i));

        if(lines.get(i).startsWith(DATE_TAG)) {
            System.out.println("Reading " + lines.get(i));
            String pole = lines.get(i).substring(DATE_TAG.length());
            if(!pole.equals("null")) {
                String[] reading = pole.split("-");
                LocalDate poleDate = LocalDate.of(Integer.parseInt(reading[0]),
                        Integer.parseInt(reading[1]), Integer.parseInt(reading[2]));
                c.setPole(poleDate);
            }
            i++;
        }

        if(lines.get(i).startsWith(WINNER_TAG)) {
            System.out.println("Reading " + lines.get(i));
            String winner = lines.get(i).substring(WINNER_TAG.length());
            c.setWinner(winner);
            i++;
        }

        for(; i < lines.size(); i++) {
            System.out.println("Reading " + lines.get(i));
            String[] reading = lines.get(i).split(":");
            c.add(reading[0],Integer.parseInt(reading[1]));
        }

        return c;
    }

    private void setPole(LocalDate d) {
        lastPole = d;
    }

    private void setWinner(String poleWinner) {
        this.poleWinner = poleWinner;
    }

    private String getUsername(Map.Entry<String, Integer> e) {
        return e.getKey();
    }

    private int getPoles(Map.Entry<String, Integer> e) {
        return e.getValue();
    }
}

class UserComparator implements Comparator<Map.Entry> {

    @Override
    public int compare(Map.Entry entr1, Map.Entry entr2) {

        return ((Integer) entr2.getValue()).compareTo((Integer) entr1.getValue());
    }
}
