package elements.messages;

import elements.exceptions.NoUsernameMessageException;
import org.json.JSONObject;

/**
 * Created by Guillermo Serrahima on 10/8/16.
 */
public class GroupMessage extends Message {
    private Long cid;         //chat id
    private Integer mid;      //message id
    private Integer uid;      //user id
    private String chatType;  //chat type
    private String fullName;  //author name
    private String userName;  //username
    private String chatName;  //chat name
    private String text;      //the actual message
    private String consoleLog;

    public GroupMessage(JSONObject message) {
        super(message);

        fullName = message.getJSONObject("from").getString("first_name");
        uid = message.getJSONObject("from").getInt("id");

        //Check for user last name
        if (message.getJSONObject("from").has("last_name"))
            fullName += " " + message.getJSONObject("from").getString("last_name");

        //Check for user username
        if (message.getJSONObject("from").has("username"))
            userName = message.getJSONObject("from").getString("username");
        else userName = null;

        //Get group chat name
        if (message.getJSONObject("chat").has("title"))
            chatName = message.getJSONObject("chat").getString("title");
        else chatName = null;

        if (message.has("text"))
            text = message.getString("text");
        else text = null;

        consoleLog = message.toString();
    }

    public long getCid() {
        return cid;
    }

    public long getMid() {
        return mid;
    }

    @Override
    public boolean hasAuthor() {
        return true;
    }

    @Override
    public int getUid() {
        return uid;
    }

    public String getAuthor() {
        return fullName;
    }

    public boolean hasUsername() {
        return userName != null;
    }

    public String getUsername() throws NoUsernameMessageException {
        if(userName == null) throw new NoUsernameMessageException();
        return userName;
    }

    @Override
    public String getChatName() {
        return chatName;
    }

    @Override
    public boolean isBasicGroup() {
        return true;
    }

    @Override
    public boolean isSupergroup() {
        return false;
    }

    @Override
    public boolean isChannel() {
        return false;
    }

    @Override
    public boolean isPrivate() {
        return false;
    }
}
