package main;

import com.mashape.unirest.http.HttpResponse;
import com.mashape.unirest.http.JsonNode;
import com.mashape.unirest.http.Unirest;
import com.mashape.unirest.http.exceptions.UnirestException;
import elements.Chat;
import elements.exceptions.NoAuthorMessageException;
import elements.exceptions.NoTextMessageException;
import elements.messages.GroupMessage;
import elements.messages.Message;
import elements.exceptions.NoChatNameMessageException;
import elements.exceptions.NoUsernameMessageException;
import elements.messages.PrivateMessage;
import org.json.JSONArray;
import org.json.JSONObject;

import java.security.PrivateKey;
import java.time.LocalDate;
import java.util.*;

/**
 * Created by Guillermo Serrahima on 4/1/16.
 */

public class Geiserbot {
    private static final String TOKEN = "267229954:AAHx49MXLmT1nT0QkccrSIzgmRVCQbjbJaQ";
    private static final String USERNAME = "geiserpbot";
    private static final String BASE_URL = "https://api.telegram.org/bot" + TOKEN;
    private static final String PARSE_MARKDOWN = "Markdown";
    private static final String PARSE_HTML = "HTML";

    private static final int WARNING_COUNT_START = 0;
    private static final int START_MESSAGE = WARNING_COUNT_START;
    private static final int HELP_MESSAGE = START_MESSAGE + 1;
    private static final int WARNING_UNKNOWN_ERROR = HELP_MESSAGE + 1;
    private static final int WARNING_NO_USERNAME = WARNING_UNKNOWN_ERROR + 1;
    private static final int WARNING_NOT_IMPLEMENTED = WARNING_NO_USERNAME + 1;
    private static final int WARNING_NO_GROUP = WARNING_NOT_IMPLEMENTED + 1;
    private static final int WARNING_COUNT_END = WARNING_NO_GROUP + 1;

    private List<String> commands;
    private Map<Long, Chat> chats;

    public Geiserbot() {

    }

    public String getUsername() {
        return USERNAME;
    }

    public String getToken() {
        return TOKEN;
    }

    public String myName() {
        return "R. Geiser Pont";
    }

    //--- Http handling

    public HttpResponse<JsonNode> sendMessage(Long chatId, String text) throws UnirestException {
        System.out.println("Sending message: \n" + text + "\n");

        return Unirest.post(BASE_URL + "/sendMessage")
                .field("chat_id", chatId)
                .field("text", text)
                .asJson();
    }

    public HttpResponse<JsonNode> sendMessage(Long chatId, String text, String parseMode) throws UnirestException {
        System.out.println("Sending message: \n" + text + "\n");

        return Unirest.post(BASE_URL + "/sendMessage")
                .field("chat_id", chatId)
                .field("text", text)
                .field("parse_mode", parseMode)
                .asJson();
    }

    public HttpResponse<JsonNode> sendMessage(String channel, String text) throws UnirestException {
        System.out.println("Sending message to " + channel + ": \n" + text + "\n");

        return Unirest.post(BASE_URL + "/sendMessage")
                .field("chat_id", channel)
                .field("text", text)
                .asJson();
    }

    public HttpResponse<JsonNode> sendMessage(String channel, String text, String parseMode) throws UnirestException {
        System.out.println("Sending message to " + channel + ": \n" + text + "\n");

        return Unirest.post(BASE_URL + "/sendMessage")
                .field("chat_id", channel)
                .field("text", text)
                .field("parse_mode", parseMode)
                .asJson();
    }

    public HttpResponse<JsonNode> replyMessage(Long chatId, Integer repliedMessage, String text) throws UnirestException {
        System.out.println("Sending message: \n" + text + "\n");

        return Unirest.post(BASE_URL + "/sendMessage")
                .field("chat_id", chatId)
                .field("text", text)
                .field("reply_to_message_id", repliedMessage)
                .asJson();
    }

    public HttpResponse<JsonNode> getUpdates(Integer offset) throws UnirestException {
        return Unirest.post(BASE_URL + "/getUpdates")
                .field("offset", offset)
                .asJson();
    }

    public HttpResponse<JsonNode> getUpdates(Integer offset, Integer timeout) throws UnirestException {
        return Unirest.post(BASE_URL + "/getUpdates")
                .field("offset", offset)
                .field("timeout", timeout)
                .asJson();
    }

    //--- bot thinking

    public void run() throws UnirestException {
        int last_upd_id = 0;
        HttpResponse<JsonNode> response;
        System.out.println("Listening.\n");

        while (true) {
            response = getUpdates(last_upd_id++);
            if (response.getStatus() == 200) {
                JSONArray responses = response.getBody().getObject().getJSONArray("result");
                if (responses.isNull(0)) continue;
                else last_upd_id = responses.getJSONObject(responses.length() - 1).getInt("update_id") + 1;

                for (int i = 0; i < responses.length(); i++) {
                    if(responses.getJSONObject(i).has("message")) {
                        JSONObject message = responses.getJSONObject(i).getJSONObject("message");

                        processMessage(message);
                    }
                }
            }
        }
    }

    private void processMessage(JSONObject message) throws UnirestException {
        //Process message into a nice and cozy java object
        Message m = null;
        try {
            m = Message.getMessage(message);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            e.printStackTrace();
        }

        //Console log
        printLog(false, m);

        //Actually read the message
        if (message.has("text")) {
            String text = message.getString("text");
            handleInput(m);
            return;
        }
    }

    private void printLog(boolean full_log, Message m) {
        if (full_log) System.out.println("\n" + m);
        else {
            String log = "\nMessage";

            try {
                log += " received from chat " + m.getChatName();
            } catch (NoChatNameMessageException e) {
                log += " received from a private chat";
            }
            log += " (chat id: " + m.getCid() + ")";
            try {
                log += " by user @" + m.getUsername();
            } catch (NoUsernameMessageException e) {
                log += " by user";
            } catch (NoAuthorMessageException ignored) {}

            try {
                log += " named " + m.getAuthor();
            } catch (NoAuthorMessageException ignored) {}

            System.out.println(log + ".");
        }
    }

    //--- Command handling

    private void handleInput(Message m) throws UnirestException {
        try {
            if (!m.isChannel()) {
                if (m.getText().startsWith("/start") &&
                        (!m.getText().substring("/start".length()).startsWith("@") ||
                                m.getText().substring("/start".length()).startsWith("@" + USERNAME)))
                    handleStart(m.getCid());

                if (m.getText().startsWith("/help") &&
                        (!m.getText().substring("/help".length()).startsWith("@") ||
                                m.getText().substring("/help".length()).startsWith("@" + USERNAME)))
                    handleHelp(m.getCid());

                if (m.getText().startsWith("/whoami") &&
                        (!m.getText().substring("/whoami".length()).startsWith("@") ||
                                m.getText().substring("/whoami".length()).startsWith("@" + USERNAME)))
                    handleWhoami(m);
            }
        } catch (NoTextMessageException ignored) {}
    }

    private void handleStart(long chat_id) throws UnirestException {
        sendMessage(chat_id, "Hola. Soy el bot personal de Guillermo Serrahima.");
        return;
    }

    private void handleHelp(long chat_id) throws UnirestException {
        String help = "Aquí tienes una lista de comandos que acepto:\n\n" +
                "/start - Repite el saludo inicial\n" +
                "/help - Saca esta lista\n" +
                "/whoami - Muestra información visible sobre ti mismo\n" +
                "\n_GeiserPBot v1.0.0_";

        sendMessage(chat_id, help, PARSE_MARKDOWN);
        return;
    }

    private void handleWhoami(Message m) throws UnirestException {
        String s = "Estamos en";

        try {
            s += " el chat " + m.getChatName();
        } catch (NoChatNameMessageException e) {
            s += " un chat";
            if(m.isPrivate()) s+= " privado";
        }
        s += " de id " + m.getCid() + ".\n";

        try {
            s += "Eres " + m.getAuthor();
            try {
                s += ", con nombre de usuario @" + m.getUsername();
            } catch (NoUsernameMessageException ignored) {}

            s += " e id de usuario " + m.getUid() + ".\n";
        } catch (NoAuthorMessageException ignored) {}

        s += "\nEn el PC que me mantiene, la fecha actual es " + LocalDate.now() +
                " en el formato ISO-8601.";

        sendMessage(m.getCid(), s);
    }

    private boolean exists(long chat_id, String user) {
        return  chats.containsKey(chat_id) && chats.get(chat_id).exists(user);
    }

    /*
    private void saveChat(long chat_id) throws SaveErrorException {
        try {
            Persistence.getInstance().saveChat(chat_id, chats.get(chat_id));
        } catch (SaveErrorException e) {
            e.printStackTrace();
            throw e;
        }
    }
    */

    private String getWarning(int warning_id) {
        switch (warning_id) {
            case WARNING_UNKNOWN_ERROR:
                return "Algo ha ido MUY mal.";

            case WARNING_NOT_IMPLEMENTED:
                return "Aún no está implementado.";

            case WARNING_NO_GROUP:
                return "Debes estar en un grupo para poder hacer eso.";

            case WARNING_NO_USERNAME:
                return "No tienes nombre de usuario. Regístrate con uno en la " +
                        "configuración de Telegram para poder utilizarme.";

            default: return "Entrada incorrecta.";
        }
    }

    private String getFormatExample(int command) {
        return "Entrada incorrecta.";
    }
}